"""emblio - python package that gets embeddings from various ML models for  different content types, like image video audio text"""
from emblio.example import test
from emblio.embedding import *
from emblio.embedding_new import *
from emblio.whisper import *

__version__ = "0.1.1"
__author__ = "Nishant Bhansali <nishantbhansali80@gmail.com>"
__all__ = []
